"""
********************************************************************************
* Name: app_base.py
* Author: Nathan Swain and Scott Christensen
* Created On: August 19, 2013
* Copyright: (c) Brigham Young University 2013
* License: BSD 2-Clause
********************************************************************************
"""


class TethysAppBase(object):
    """
    Base class used for building apps
    """
    name = ''
    index = ''
    icon = ''
    root_url = ''
    color = ''

    def __repr__(self):
        """
        String representation
        """
        return '<TethysApp: {0}>'.format(self.name)

    def controllers(self, app):
        """
        Must return a list of AppController objects
        """
        raise NotImplementedError()
    
    def persistent_stores(self, controllers):
        """
        Should return a list of PersistentStore objects
        """
        pass


"""
********************************************************************************
* Name: persistent store
* Author: Nathan Swain
* Created On: September 22, 2014
* Copyright: (c) Brigham Young University 2014
* License: BSD 2-Clause
********************************************************************************
"""

import sys

from django.conf import settings
from sqlalchemy import create_engine


class PersistentStore(object):
    """
    An object that stores the data for a Tethys Persistent store
    """

    def __init__(self, name, initializer, postgis=False):
        """
        Constructor
        """
        # Validate
        ## TODO: Validate persistent store object
        self.name = name
        self.initializer = initializer
        self.postgis = postgis

    def __repr__(self):
        """
        String representation
        """
        return '<Persistent Store: name={0}, initializer={1}, postgis={2}>'.format(self.name,
                                                                                   self.initializer,
                                                                                   self.postgis)


def get_persistent_store_engine(app_name, persistent_store_name):
    """
    Returns an sqlalchemy engine for the given store
    """
    # Create the unique store name
    unique_store_name = '_'.join([app_name, persistent_store_name])

    # Get database manager
    database_manager_url = settings.TETHYS_APPS_DATABASE_MANAGER_URL

    # Create connection engine
    engine = create_engine(database_manager_url)
    connection = engine.connect()

    # Check for Database
    existing_dbs_statement = '''
                             SELECT d.datname as name
                             FROM pg_catalog.pg_database d
                             LEFT JOIN pg_catalog.pg_user u ON d.datdba = u.usesysid
                             ORDER BY 1;
                             '''

    existing_dbs = connection.execute(existing_dbs_statement)

    # Compile list of db names
    existing_db_names = []

    for existing_db in existing_dbs:
        existing_db_names.append(existing_db.name)

    # Check to make sure that the persistent store exists
    if unique_store_name in existing_db_names:
        # Retrieve the database manager url.
        # The database manager database user is the owner of all the app databases.
        database_manager_url = settings.TETHYS_APPS_DATABASE_MANAGER_URL
        url_parts = database_manager_url.split('/')

        # Assemble url for persistent store with that name
        persistent_store_url = '{0}//{1}/{2}'.format(url_parts[0], url_parts[2], unique_store_name)

        # Return SQLAlchemy Engine
        return create_engine(persistent_store_url)

    else:
        print('ERROR: No persistent store "{0}" for app "{1}". Make sure you register the persistent store in app.py '
              'and reinstall app.'.format(persistent_store_name, app_name))
        sys.exit()


import os
import shutil
import subprocess

from setuptools.command.develop import develop
from setuptools.command.install import install


def get_tethysapp_directory():
    """
    Return the absolute path to the tethysapp directory.
    """
    return os.path.join(os.path.abspath(os.path.dirname(__file__)), 'tethysapp')


def _run_install(self):
    """
    The definition of the "run" method for the CustomInstallCommand metaclass.
    """
    # Get paths
    tethysapp_dir = get_tethysapp_directory()
    destination_dir = os.path.join(tethysapp_dir, self.app_package)

    # Notify user
    print('Copying App Package: {0} to {1}'.format(self.app_package_dir, destination_dir))

    # Copy files
    try:
        shutil.copytree(self.app_package_dir, destination_dir)

    except:
        try:
            shutil.rmtree(destination_dir)
        except:
            os.remove(destination_dir)

        shutil.copytree(self.app_package_dir, destination_dir)

    # Install dependencies
    for dependency in self.dependencies:
        subprocess.call(['pip', 'install', dependency])

    # Run the original install command
    install.run(self)


def _run_develop(self):
    """
    The definition of the "run" method for the CustomDevelopCommand metaclass.
    """
    # Get paths
    tethysapp_dir = get_tethysapp_directory()
    destination_dir = os.path.join(tethysapp_dir, self.app_package)

    # Notify user
    print('Creating Symbolic Link to App Package: {0} to {1}'.format(self.app_package_dir, destination_dir))

    # Create symbolic link
    try:
        os.symlink(self.app_package_dir, destination_dir)

    except:
        try:
            shutil.rmtree(destination_dir)
        except:
            os.remove(destination_dir)

        os.symlink(self.app_package_dir, destination_dir)

    # Install dependencies
    for dependency in self.dependencies:
        subprocess.call(['pip', 'install', dependency])

    # Run the original develop command
    develop.run(self)


def custom_install_command(app_package, app_package_dir, dependencies):
    """
    Returns a custom install command class that is tailored for the app calling it.
    """
    # Define the properties (and methods) for the class that will be created.
    properties = {'app_package': app_package,
                  'app_package_dir': app_package_dir,
                  'dependencies': dependencies,
                  'run': _run_install}

    return type('CustomInstallCommand', (install, object), properties)


def custom_develop_command(app_package, app_package_dir, dependencies):
    """
    Returns a custom develop command class that is tailored for the app calling it.
    """
    # Define the properties (and methods) for the class that will be created.
    properties = {'app_package': app_package,
                  'app_package_dir': app_package_dir,
                  'dependencies': dependencies,
                  'run': _run_develop}

    return type('CustomDevelopCommand', (develop, object), properties)


from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, Float, String
from sqlalchemy.orm import sessionmaker

from .utilities import get_persistent_store_engine

# DB Engine, sessionmaker, and base
engine = get_persistent_store_engine('primary')
SessionMaker = sessionmaker(bind=engine)
Base = declarative_base()


class StreamGage(Base):
    """
    Example SQLAlchemy model
    """
    __tablename__ = 'stream_gages'

    # Columns
    id = Column(Integer, primary_key=True)
    name = Column(String)
    lat = Column(Float)
    lon = Column(Float)